/** @file sp/spOption.h
 */

#ifndef __SPOPTION_H
#define __SPOPTION_H

#include <stdarg.h>

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spOptionGroup Command-Line Options */

/** @addtogroup spOptionGroup
 *  @{ */

#define SP_USAGE_LABEL_STRING "???"
    
#define SP_MAX_SETUP_LINE         512
#define SP_MAX_SETUP_NAME         128
/*#define SP_MAX_SETUP_VALUE         256*/
#define SP_MAX_SETUP_VALUE         8192

#define SP_UNKNOWN         (-1)

#define SP_MAX_EXIT_CALLBACK         16
typedef void (*spExitFunc)(int status);

typedef void (*spExitCallbackFunc)(void *);  /**< Definition of spExitCallbackFunc used for spAddExitCallback() . */
    
typedef unsigned short spOptionType;  /**< Definition of spOptionType . */

#define SP_TYPE_NONE	0
#define SP_TYPE_BOOL    1
#define SP_TYPE_STRING  2
#define SP_TYPE_INT     3
#define SP_TYPE_SHORT   4
#define SP_TYPE_LONG    5
#define SP_TYPE_FLOAT   6
#define SP_TYPE_DOUBLE  7
#define SP_TYPE_CHAR    8
#define SP_TYPE_STRING_A 9  /* string array */
#define SP_TYPE_ENUM    10
#define SP_TYPE_USER    16
#define SP_TYPE_BOOLEAN SP_TYPE_BOOL
#define SP_TYPE_STRING_S SP_TYPE_STRING_A

#define SP_TYPE_EDITABLE (1<<8)
#define SP_TYPE_LIST (1<<9)
#define SP_TYPE_COMBO (1<<10)
#define SP_TYPE_TOGGLE (1<<11)
#define SP_TYPE_SHOW_TIP (1<<12)
#define SP_TYPE_NO_GUI (1<<13)
#define SP_TYPE_INPUT_ONLY (1<<14) /* 0.8.19+ */
#define SP_TYPE_OUTPUT_ONLY (1<<15) /* 0.8.19+ */

#define SpNullOption(flag, subflag, type) {flag, subflag, NULL, NULL, type, NULL, NULL}

typedef struct _spOptions *spOptions;  /**< Definition of spOptions. */

/** @struct spOption spOption.h <sp/spOption.h>
 *  A structure to hold parameters for a command-line option.
 */
typedef struct _spOption {
    const char *flag;           /**< Option flag. */
    const char *subflag;        /**< Option subflag. */
    const char *desc;           /**< Description for help. */
    const char *label;          /**< Label for the setup file. */
    spOptionType type;          /**< Type of value. */
    void *value;                /**< Option value. */
    const char *def_value;      /**< Default value. */
} spOption;

#if defined(MACOS)
#pragma import on
#endif

/** @~english Checks if two option types are the same. */
extern spBool spEqType(spOptionType type1, spOptionType type2);
extern void spIgnoreUnknownOption(spOptions options, spBool flag);
extern int spGetNumOption(spOptions options);
extern spOption *spGetOptionList(spOptions options);
extern char *xspCutOptionValue(const char *value, int index);
extern int spConvertOptionValue(spOption *option, const char *value);
extern spOptions spAllocOptions(int num_option, spOption *option_list);
extern spOptions spCopyOptions(int num_option, spOption *option_list);
extern spOptions spInitOptions(int num_option, spOption *option_list);
extern void _spFreeOptionsAndValues(spOptions options, spBool free_values);
extern void _spFreeOptions(spOptions options);
extern spOptions _spGetOptions(int argc, char **argv, int num_option, spOption *option_list,
                               int num_file, const char **labels);
/** @~english Sets the section number for a help message. */
extern void spSetSection(spOptions options, int section);
/** @~english Sets the file name of the setup file. */
extern void spSetSetup(const char *filename);
/** @~english Writes the contents of the setup file. */
extern spBool spWriteGlobalSetup(void);
/** @~english Adds a callback function called at exit. */
extern spBool spAddExitCallback(spExitCallbackFunc callback, void *data);
/** @~english Removes a callback function added by spAddExitCallback() . */
extern spBool spRemoveExitCallback(spExitCallbackFunc callback, void *data);
/** @~english Terminates the current program. */
extern void spExit(int status);
/** @~english Parses a command-line option. This function parses only one option. Use spGetOptionsValue() to parse multiple options. */
extern char *spGetOptionValue(int argc, char **argv, spOptions options);
/** @~english Parses multiple command-line options. This function calls spGetOptionValue() internally. */
extern void spGetOptionsValue(int argc, char **argv, spOptions options);
extern void spUpdateOptionsValue(int argc, char **argv, spOptions options);
/** @~english Checks if the number of files in the command line is correct. */
extern void spCheckNumFile(spOptions options);
/** @~english Gets the file name specified in the command line one by one. */
extern const char *spGetFile(spOptions options);
extern char *xspGetOptionLabel(spOption *option, int index);
extern void spSetOptionDescOffset(int offset);
/** @~english Sets the contents of a help message. */
extern void spSetHelpMessage(spBool *flag, const char *format, ...);
extern void spPrintHelpHeader(const char *format, ...);
/** @~english Displays a help message. */
extern void spPrintHelp(const char *format, ...);
/** @~english Displays an error message. */
extern void spPrintError(const char *format, ...);
/** @~english Generates a text in man format. */
extern void spPrintMan(spOptions options, const char *description, const char *author);
/** @~english Reads settings from the setup file. */
extern spBool spReadSetup(const char *filename, spOptions options);
/** @~english Writes settings to the setup file. */
extern spBool spWriteSetup(const char *filename, spOptions options);
extern spBool spGetOptionValueString(spOption *option, char *buf, int buf_size); /* 0.8.18+ */
extern spBool spPrintOption(spOption *option);
extern spBool spPrintOptions(spOptions options);
extern void spPrintUsageHeader(void);
extern void spPrintUsage(void);
/** @~english Displays a help message and then terminates the current program. */
extern void spUsage(void);

/** @} */  /*----@addtogroup spOptionGroup----*/

#if defined(_WIN32)
extern void spSetHInstanceWin(void *hinstance, unsigned long reason);
extern void *spGetHInstanceWin(void);
#endif
    
/* Do not use the following function. This is for internal use. */
extern spBool spSetExitFunc(spExitFunc func);
extern spBool spEmitExitCallback(void);

#if defined(MACOS)
#pragma import off
#endif

/** @addtogroup spOptionGroup
 *  @{ */  /*----@addtogroup spOptionGroup----*/

#define spFreeOptions(options) {_spFreeOptions(options); (options) = NULL;}
#define spFreeOptionsAndValues(options, free_values) {_spFreeOptionsAndValues(options, free_values); (options) = NULL;}
/** @~english Initializes #spOptions which is a structure to deal with command-line options. */
#define spGetOptions(argc, argv, option, file_label) \
_spGetOptions(argc, argv, spArraySize(option), option, spArraySize(file_label), file_label)
    
/*#define spEqType(type1, type2) (((0xff & type1) == (0xff & type2)) ? 1 : 0)*/
/** @~english Converts a null-terminated string to a boolean value. */
#define spStrToBool(value) (((*(value) == 'T') || spStrEq(value, "1") || spStrEq(value, "ON") || spStrEq(value, "On")) ? SP_TRUE : SP_FALSE)
/** @~english Converts a boolean value to a null-terminated string. */
#define spBoolToStr(value) (((value) == SP_TRUE) ? "True" : "False")

/** @} */  /*----@addtogroup spOptionGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPOPTION_H */
