/** @file sp/spPSFile.h
 */

#ifndef __SPPSFILE_H
#define __SPPSFILE_H

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SP_PS_DEFAULT 0L
#define SP_PS_MONO (1L<<1)	/* monochrome output */
#define SP_PS_LANDSCAPE (1L<<2)	/* use langscape mode instead of portrait mode */
#define SP_PS_EPS (1L<<3) 	/* EPS file */
#define SP_PS_HALF_SIZE (1L<<4)	/* use half size output. margin will not be half */
#define SP_PS_SCREEN_COORDINATE (1L<<5)	/* use screen coordinate instead of Cartesian coordinate */
#define SP_PS_DOUBLE_SIDED (1L<<8)	/* use double-sided printing */
#define SP_PS_TUMBLE (1L<<9)		/* tumble option for double-sided printing */

typedef enum {
    SP_PS_LINE_NONE = -1,
    SP_PS_LINE_SOLID = 0,
    SP_PS_LINE_DASH = 1,
    SP_PS_LINE_DOT = 2,
    SP_PS_LINE_DASH_DOT = 3,
    SP_PS_LINE_DASH_DOT_DOT = 4,
    SP_PS_LINE_BORDER = 10,
    SP_PS_LINE_GRID = 11,
} spPSLineType;

typedef enum {
    SP_PS_ALIGNMENT_BEGINNING = 0,
    SP_PS_ALIGNMENT_CENTER = 1,
    SP_PS_ALIGNMENT_END = 2,
} spPSAlignment;

typedef long spPSPolygonId;
#define SP_PS_POLYGON_NONE 0L

    
#define SP_PS_DEFAULT_CREATOR "spPSFile"
#define SP_PS_DEFAULT_LANG "C"
#define SP_PS_DEFAULT_FONT "Helvetica"
#define SP_PS_DEFAULT_FONT_SIZE 14
#define SP_PS_DEFAULT_MARGIN_MM 20 /* default margin is 20 mm (for left and bottom) */

/* the following fonts are used when lang by spPSSetLang is
   Japanese one such as "ja_JP.SJIS" and "ja_JP.eucJP" */    
#define SP_PS_DEFAULT_SJIS_FONT "Ryumin-Light-RKSJ-H"
#define SP_PS_DEFAULT_SJIS_GOTHIC_FONT "GothicBBB-Medium-RKSJ-H"
#define SP_PS_DEFAULT_EUCJP_FONT "Ryumin-Light-EUC-H"
#define SP_PS_DEFAULT_EUCJP_GOTHIC_FONT "GothicBBB-Medium-EUC-H"
#define SP_PS_DEFAULT_UTF8JP_FONT "Ryumin-Light-UniJIS-UTF8-H"
#define SP_PS_DEFAULT_UTF8JP_GOTHIC_FONT "GothicBBB-Medium-UniJIS-UTF8-H"
    
typedef struct _spPSFile spPSFile;
    
#if defined(MACOS)
#pragma import on
#endif

/* conversion between milli meter <--> point */
extern double spPSMMToPoint(double mm);
extern double spPSPointToMM(double point);

/* width and height here do not include margins */    
extern spPSFile *spPSOpenFile(const char *filename, double client_width_mm, double client_height_mm, unsigned long options);
extern spBool spPSCloseFile(spPSFile *psfile);

/* following four functions must be called before drawing */    
extern spBool spPSSetNumCopies(spPSFile *psfile, int num_copies, spBool collate);
extern spBool spPSSetScalingFactor(spPSFile *psfile, double xfactor, double yfactor);
extern spBool spPSSetMargin(spPSFile *psfile, double left_margin_mm, double bottom_margin_mm);
extern spBool spPSSetCreator(spPSFile *psfile, const char *creator);
    
extern spBool spPSGetClientSize(spPSFile *psfile, int *width, int *height);
extern spBool spPSGetPhisicalSize(spPSFile *psfile, double *client_width_mm, double *client_height_mm);
extern spBool spPSStartDrawing(spPSFile *psfile);
extern spBool spPSEndDrawing(spPSFile *psfile);
extern spBool spPSForwardPage(spPSFile *psfile);
    
extern spBool spPSSetLang(spPSFile *psfile, const char *lang);
extern spBool spPSSetLineType(spPSFile *psfile, spPSLineType line_type);
extern spBool spPSSetLineWidth(spPSFile *psfile, int width);
extern spBool spPSSetRGBColor(spPSFile *psfile, double r, double g, double b);
extern spBool spPSSetFontSize(spPSFile *psfile, int size);
extern int spPSGetFontSize(spPSFile *psfile);
extern spBool spPSSetFont(spPSFile *psfile, const char *font); /* font: XLFD */
extern spBool spPSSetFontFamily(spPSFile *psfile, const char *font_family);
extern const char *spPSGetFontFamily(spPSFile *psfile);
extern spBool spPSSetStringAlignment(spPSFile *psfile, spPSAlignment alignment);
extern spBool spPSSetStringRotation(spPSFile *psfile, int angle);
    
extern spBool spPSMoveTo(spPSFile *psfile, int x, int y);
extern spBool spPSDrawLineTo(spPSFile *psfile, int x, int y);
extern spBool spPSDrawLine(spPSFile *psfile, int x1, int y1, int x2, int y2);
extern spBool spPSDrawString(spPSFile *psfile, int x, int y, const char *string);

/* screen coordinate case: (x, y) = upper left corner
   cartesian coordinate case: (x, y) = lower left corder */
extern spBool spPSFillRectangle(spPSFile *psfile, int x, int y, int width, int height);
extern spBool spPSDrawRectangle(spPSFile *psfile, int x, int y, int width, int height);
extern spBool spPSFillArc(spPSFile *psfile, int x, int y, int width, int height, double angle1, double angle2);
extern spBool spPSDrawArc(spPSFile *psfile, int x, int y, int width, int height, double angle1, double angle2);
    
extern spBool spPSStartPolygonPoint(spPSFile *psfile, int x, int y);
extern spBool spPSAddPolygonPoint(spPSFile *psfile, int x, int y);
extern spBool spPSAddPolygonRelativePoint(spPSFile *psfile, int dx, int dy);
extern spPSPolygonId spPSEndPolygonPoint(spPSFile *psfile);
extern spBool spPSFillPolygon(spPSFile *psfile, spPSPolygonId polygon_id, int dx, int dy);
extern spBool spPSDrawPolygon(spPSFile *psfile, spPSPolygonId polygon_id, int dx, int dy);
    
#if defined(MACOS)
#pragma import off
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPSFILE_H */
